% This file is part of the OPTPDE problem collection
%                 http://www.optpde.net
% It is provided as is, without any warranty, under the 
%
% Creative Commons Attribution-NonCommercial ShareAlike 4.0 International
%
%         http://creativecommons.org/licenses/by-nc-sa/4.0/
%
% Please give proper credits when using this file by referencing the 
% collection. For this purpose detailed bibliography data is 
% available at the collections website 
%                  
%                 http://www.optpde.net
%
% This file contains data for the benchmark problem 
%                      mpccdist2
% from  2022-04-01 


% setup the problem
problem = setup_problem;

% generate a grid on the domain
[r,phi] = meshgrid(linspace(0,1,40),linspace(0,2*pi,41));
[x1,x2] = pol2cart(phi,r);

% plot the desired displacement component 1
figure(1); clf; hold on
val = uOmega1(problem,x1,x2);
surf(x1,x2,val);
xlabel('x_1');
ylabel('x_2');
title('desired displacement u_{\Omega,1}');
view(3);

% plot the desired displacement component 2
figure(2); clf; hold on
val = uOmega2(problem,x1,x2);
surf(x1,x2,val);
xlabel('x_1');
ylabel('x_2');
title('desired state u_{\Omega,2}');
view(3);

% plot the desired control component 1
figure(3); clf; hold on
val = fOmega1(problem,x1,x2);
surf(x1,x2,val);
xlabel('x_1');
ylabel('x_2');
title('desired control f_{\Omega,1}');
view(3);

% plot the desired control component 2
figure(4); clf; hold on
val = fOmega2(problem,x1,x2);
surf(x1,x2,val);
xlabel('x_1');
ylabel('x_2');
title('desired control f_{\Omega,2}');
view(3);

% plot the optimal displacement component 1
figure(5); clf; hold on
val = supplementary_u1(problem,x1,x2);
surf(x1,x2,val);
xlabel('x_1');
ylabel('x_2');
title('optimal displacement u_1');
view(3);

% plot the optimal displacement component 2
figure(6); clf; hold on
val = supplementary_u2(problem,x1,x2);
surf(x1,x2,val);
xlabel('x_1');
ylabel('x_2');
title('optimal displacement u_2');
view(3);

% show the absolute value of the optimal displacement
figure(7); clf; hold on
val = sqrt(supplementary_u1(problem,x1,x2).^2+supplementary_u1(problem,x1,x2).^2);
surf(x1,x2,val);
xlabel('x_1');
ylabel('x_2');
title('absolute value of optimal displacement u');
view(2);

% plot the optimal strain component 11
figure(8); clf; hold on
val = supplementary_epsilon11(problem,x1,x2);
surf(x1,x2,val);
xlabel('x_1');
ylabel('x_2');
title('optimal strain \epsilon_{11}');
view(3);

% plot the optimal strain component 12
figure(9); clf; hold on
val = supplementary_epsilon12(problem,x1,x2);
surf(x1,x2,val);
xlabel('x_1');
ylabel('x_2');
title('optimal strain \epsilon_{12}');
view(3);

% plot the optimal strain component 22
figure(10); clf; hold on
val = supplementary_epsilon22(problem,x1,x2);
surf(x1,x2,val);
xlabel('x_1');
ylabel('x_2');
title('optimal strain \epsilon_{22}');
view(3);

% plot the optimal control component 1
figure(11); clf; hold on
val = supplementary_f1(problem,x1,x2);
surf(x1,x2,val);
xlabel('x_1');
ylabel('x_2');
title('optimal control f_1');
view(3);

% plot the optimal control component 2
figure(12); clf; hold on
val = supplementary_f2(problem,x1,x2);
surf(x1,x2,val);
xlabel('x_1');
ylabel('x_2');
title('optimal control f_2');
view(3);

% show the absolute value of the optimal control
figure(13); clf; hold on
val = sqrt(supplementary_f1(problem,x1,x2).^2+supplementary_f1(problem,x1,x2).^2);
surf(x1,x2,val);
xlabel('x_1');
ylabel('x_2');
title('absolute value of optimal control f');
view(2);

% plot the optimal adjoint stress component 11
figure(14); clf; hold on
val = supplementary_zeta11(problem,x1,x2);
surf(x1,x2,val);
xlabel('x_1');
ylabel('x_2');
title('optimal adjoint stress \zeta_{11}');
view(3);

% plot the optimal adjoint stress component 12
figure(15); clf; hold on
val = supplementary_zeta12(problem,x1,x2);
surf(x1,x2,val);
xlabel('x_1');
ylabel('x_2');
title('optimal adjoint stress \zeta_{12}');
view(3);

% plot the optimal adjoint stress component 22
figure(16); clf; hold on
val = supplementary_zeta22(problem,x1,x2);
surf(x1,x2,val);
xlabel('x_1');
ylabel('x_2');
title('optimal adjoint stress \zeta_{22}');
view(3);

% plot the optimal adjoint back stress component 11
figure(17); clf; hold on
val = supplementary_psi11(problem,x1,x2);
surf(x1,x2,val);
xlabel('x_1');
ylabel('x_2');
title('optimal adjoint back stress \psi_{11}');
view(3);

% plot the optimal adjoint back stress component 12
figure(18); clf; hold on
val = supplementary_psi12(problem,x1,x2);
surf(x1,x2,val);
xlabel('x_1');
ylabel('x_2');
title('optimal adjoint back stress \psi_{12}');
view(3);

% plot the optimal adjoint back stress component 22
figure(19); clf; hold on
val = supplementary_psi22(problem,x1,x2);
surf(x1,x2,val);
xlabel('x_1');
ylabel('x_2');
title('optimal adjoint back stress \psi_{22}');
view(3);

% plot the multiplier mu
figure(20); clf; hold on
val = supplementary_mu(problem,x1,x2);
surf(x1,x2,val);
xlabel('x_1');
ylabel('x_2');
title('optimal multiplier \mu');
view(3);

% plot the multiplier theta
figure(21); clf; hold on
val = supplementary_theta(problem,x1,x2);
surf(x1,x2,val);
xlabel('x_1');
ylabel('x_2');
title('optimal multiplier \theta');
view(3);

% show the optimal yield function
figure(22); clf; hold on
val = supplementary_phi(problem,x1,x2);
surf(x1,x2,val);
xlabel('x_1');
ylabel('x_2');
title('optimal yield function \phi');
view(3);

% show the plastic zone
figure(23); clf; hold on
val = double(supplementary_phi(problem,x1,x2) > -1e-4);
surf(x1,x2,val);
xlabel('x_1');
ylabel('x_2');
title('plastic zone');
view(2);
axis equal

% Check equation for adjoint stress zeta
r_zeta11 = max(max(abs(supplementary_zeta11(problem,x1,x2) - 2 * supplementary_epsilon11(problem,x1,x2) + 0 * (supplementary_zetaD11(problem,x1,x2) + supplementary_psiD11(problem,x1,x2)) + supplementary_theta(problem,x1,x2) .* (supplementary_sigmaD11(problem,x1,x2) + 0))));
r_zeta12 = max(max(abs(supplementary_zeta12(problem,x1,x2) - 2 * supplementary_epsilon12(problem,x1,x2) + 0 * (supplementary_zetaD12(problem,x1,x2) + supplementary_psiD12(problem,x1,x2)) + supplementary_theta(problem,x1,x2) .* (supplementary_sigmaD12(problem,x1,x2) + 0))));
r_zeta21 = max(max(abs(supplementary_zeta21(problem,x1,x2) - 2 * supplementary_epsilon21(problem,x1,x2) + 0 * (supplementary_zetaD21(problem,x1,x2) + supplementary_psiD21(problem,x1,x2)) + supplementary_theta(problem,x1,x2) .* (supplementary_sigmaD21(problem,x1,x2) + 0))));
r_zeta22 = max(max(abs(supplementary_zeta22(problem,x1,x2) - 2 * supplementary_epsilon22(problem,x1,x2) + 0 * (supplementary_zetaD22(problem,x1,x2) + supplementary_psiD22(problem,x1,x2)) + supplementary_theta(problem,x1,x2) .* (supplementary_sigmaD22(problem,x1,x2) + 0))));
r_zeta = [r_zeta11, r_zeta12; r_zeta21, r_zeta22];
fprintf('Adjoint stress equation residual:      %8.2e...',norm(r_zeta));
if (norm(r_zeta) < 1e-12)
	fprintf('PASSED\n');
else
	fprintf('FAILED\n');
end

% Check equation for adjoint back stress psi
r_psi11 = max(max(abs(supplementary_psi11(problem,x1,x2) + 0 * (supplementary_zetaD11(problem,x1,x2) + supplementary_psiD11(problem,x1,x2)) + supplementary_theta(problem,x1,x2) .* (supplementary_sigmaD11(problem,x1,x2) + 0))));
r_psi12 = max(max(abs(supplementary_psi12(problem,x1,x2) + 0 * (supplementary_zetaD12(problem,x1,x2) + supplementary_psiD12(problem,x1,x2)) + supplementary_theta(problem,x1,x2) .* (supplementary_sigmaD12(problem,x1,x2) + 0))));
r_psi21 = max(max(abs(supplementary_psi21(problem,x1,x2) + 0 * (supplementary_zetaD21(problem,x1,x2) + supplementary_psiD21(problem,x1,x2)) + supplementary_theta(problem,x1,x2) .* (supplementary_sigmaD21(problem,x1,x2) + 0))));
r_psi22 = max(max(abs(supplementary_psi22(problem,x1,x2) + 0 * (supplementary_zetaD22(problem,x1,x2) + supplementary_psiD22(problem,x1,x2)) + supplementary_theta(problem,x1,x2) .* (supplementary_sigmaD22(problem,x1,x2) + 0))));
r_psi = [r_psi11, r_psi12; r_psi21, r_psi22];
fprintf('Adjoint back stress equation residual: %8.2e...',norm(r_psi));
if (norm(r_psi) < 1e-12)
	fprintf('PASSED\n');
else
	fprintf('FAILED\n');
end

% Check relation between optimal control and adjoint displacement
r_f1 = max(max(abs(supplementary_w1(problem,x1,x2) + problem.alpha * (supplementary_f1(problem,x1,x2) - fOmega1(problem,x1,x2)))));
r_f2 = max(max(abs(supplementary_w2(problem,x1,x2) + problem.alpha * (supplementary_f2(problem,x1,x2) - fOmega2(problem,x1,x2)))));
r_f = [r_f1; r_f2];
fprintf('Gradient equation residual:            %8.2e...',norm(r_f));
if (norm(r_f) < 1e-12)
	fprintf('PASSED\n');
else
	fprintf('FAILED\n');
end

% Check equation for multiplier mu
r_mu = max(max(abs( ...
+ (supplementary_sigmaD11(problem,x1,x2) + 0) .* (supplementary_zetaD11(problem,x1,x2) + supplementary_psiD11(problem,x1,x2)) ...
+ (supplementary_sigmaD12(problem,x1,x2) + 0) .* (supplementary_zetaD12(problem,x1,x2) + supplementary_psiD12(problem,x1,x2)) ...
+ (supplementary_sigmaD21(problem,x1,x2) + 0) .* (supplementary_zetaD21(problem,x1,x2) + supplementary_psiD21(problem,x1,x2)) ...
+ (supplementary_sigmaD22(problem,x1,x2) + 0) .* (supplementary_zetaD22(problem,x1,x2) + supplementary_psiD22(problem,x1,x2)) ...
- supplementary_mu(problem,x1,x2) ...
)));
fprintf('Multiplier mu equation residual:       %8.2e...',norm(r_mu));
if (norm(r_mu) < 1e-12)
	fprintf('PASSED\n');
else
	fprintf('FAILED\n');
end

% The complementarity between mu and lambda
% holds trivially in view of lambda = 0
r_complementarity1 = 0;
fprintf('Complementarity condition #1:           TRIVIAL...');
fprintf('PASSED\n');

% Check complementarity between theta and yield function phi
r_complementarity2 = max(max(abs(supplementary_theta(problem,x1,x2) .* supplementary_phi(problem,x1,x2))));
fprintf('Complementarity condition #2:          %8.2e...',norm(r_complementarity2));
if (norm(r_complementarity2) < 1e-12)
	fprintf('PASSED\n');
else
	fprintf('FAILED\n');
end

